// api.controller.js

// Cargamos el schema del modelo de datos para poder usarlo dentro de la lógica del controlador
// y así poder añadir, borrar, etc. sobre productos.
const Producto = require('../models/productos.model');

// Prueba a una ruta /api/v1/productos/test
exports.api_test = function (req, res) {
    res.send('Saludos desde el controlador de pruebas /api/v1/productos/test !');
};


exports.api_productos_create = function (req, res) {
    let producto = new Producto(
    {
        nombre: req.body.nombre,
        precio: req.body.precio
    }
    );

    producto.save(function (err) {
        if (err)
        {
            // Si se ha producido un error, salimos de la función devolviendo  código http 422 (Unprocessable Entity).
            return (res.type('json').status(422).send({ status: "error", data: "No se puede procesar la entidad, datos incorrectos!" }));
        }

      // Enviamos al cliente la siguiente respuesta con el código HTTP 200.
      res.type('json').status(200).send({ status: "ok", data: "Producto creado satisfactoriamente!" });
  })
};


exports.api_productos_read = function (req, res) {
    // Si a esta ruta le pasamos una cadena de consulta (query string) del estilo ?busqueda=xxxxx realiza la búsqueda de productos cuyo nombre contenga xxxx.
    // Con req.query tenemos acceso a las variables recibidas por GET.
    if (req.query.busqueda) // Si ?busqueda contiene algún texto.
    {
        // Referencia para este tipo de búsqueda en:
        // https://stackoverflow.com/questions/8246019/case-insensitive-search-in-mongo
        Producto.find({nombre:{'$regex' : req.query.busqueda, '$options' : 'i'}}, function (err, productos) {
            if (productos.length==0)
            {
                // Devolvemos el código HTTP 404, de producto no encontrado por su id.
                res.status(404).json({ status: "error", data: "No se ha encontrado ningún producto cuyo nombre contenga: "+req.query.busqueda});
            }
            else
            {
                // También podemos devolver así la información:
                res.status(200).json({ status: "ok", total:productos.length, data: productos });
            }
        })
    }
    else if (req.params.id)
    {
     // Con req.params tenemos acceso a los parámetros extraídos de la ruta.
     // Si le pasamos un ID de producto devuelve los datos de ese producto
     Producto.findById(req.params.id, function (err, unProducto) {
        if (err)
        {
            // Devolvemos el código HTTP 404, de producto no encontrado por su id.
            res.status(404).json({ status: "error", data: "No se ha encontrado el producto con id: "+req.params.id});
        }
        else
        {
            // También podemos devolver así la información:
            res.status(200).json({ status: "ok", data: unProducto });
        }
    })
 }
 else
 {
   // En otro caso devolverá todos los productos
   // Ayuda: https://mongoosejs.com/docs/api/model.html
   Producto.find({}, function (err, todosProductos) {
    if (err)
        // Si se ha producido un error, salimos de la función devolviendo  código http 422 (Unprocessable Entity).
        return (res.type('json').status(422).send({ status: "error", data: "No se puede procesar la entidad, datos incorrectos!" }));

  // También podemos devolver así la información:
  res.status(200).json({ status: "ok", data: todosProductos });
})
}
};


exports.api_productos_update = function (req, res) {
    Producto.findByIdAndUpdate(req.params.id, { $set: req.body }, function (err, productoActualizado) {
        if (err)
        {
            //res.send(err);
            // Devolvemos el código HTTP 404, de producto no encontrado por su id.
            res.status(404).json({ status: "error", data: "No se ha encontrado el producto con id: "+req.params.id});
        }
        else
        {
            // Devolvemos el código HTTP 200.
            res.status(200).json({ status: "ok", data: productoActualizado });
        }
    });
};


exports.api_productos_delete = function (req, res) {
    Producto.findByIdAndRemove(req.params.id, function(err, data) {
        if (err || !data) {
            //res.send(err);
            // Devolvemos el código HTTP 404, de producto no encontrado por su id.
            res.status(404).json({ status: "error", data: "No se ha encontrado el producto con id: "+req.params.id});
        }
        else
        {
            // Devolvemos el código HTTP 204 cuando lo ha borrado (No Content). No devuelve contenido.
            res.status(204).json({ status: "ok", data: "Se ha eliminado correctamente el producto con id: "+req.params.id});
        }
    });
};
